#include "config.h"

#include "xmms/i18n.h"
#include <xmms/plugin.h>
#include <glib.h>
#include "libxmms/util.h"
#include "libxmms/configfile.h"

static void init(void);
static void about(void);
static void configure(void);
static int mod_samples(gpointer *d, gint length, AFormat afmt, gint srate, gint nch);

EffectPlugin stereo_ep =
{
	NULL,
	NULL,
	NULL, /* Description */
	init,
	NULL,
	about,
	configure,
	mod_samples
};

static const char *about_text = N_("Extra Stereo Plugin\n\n"
				   "By Johan Levin 1999.");

static void *conf_dialog = NULL;
static gfloat value;

EffectPlugin *get_eplugin_info(void)
{
	stereo_ep.description =
		g_strdup_printf(_("Extra Stereo Plugin %s"), VERSION);
	return &stereo_ep;
}

static void init(void)
{
	ConfigFile *cfg;
	cfg = xmms_cfg_open_default_file();
	if (!xmms_cfg_read_float(cfg, "extra_stereo", "intensity", &value))
		value = 2.5;
	xmms_cfg_free(cfg);
}

static void about(void)
{}

static void conf_ok_cb(void * button, gpointer data)
{}

static void conf_cancel_cb(void * button, gpointer data)
{}

static void conf_apply_cb(void *button, gpointer data)
{}

static void configure(void)
{}

static int mod_samples(gpointer *d, gint length, AFormat afmt, gint srate, gint nch)
{
	gint i;
	gdouble avg, ldiff, rdiff, tmp, mul;
	gint16  *data = (gint16 *)*d;

	if (!(afmt == FMT_S16_NE ||
	      (afmt == FMT_S16_LE && G_BYTE_ORDER == G_LITTLE_ENDIAN) ||
	      (afmt == FMT_S16_BE && G_BYTE_ORDER == G_BIG_ENDIAN)) ||
	    nch != 2)
		return length;

	mul = value;
	
	for (i = 0; i < length / 2; i += 2)
	{
		avg = (data[i] + data[i + 1]) / 2;
		ldiff = data[i] - avg;
		rdiff = data[i + 1] - avg;

		tmp = avg + ldiff * mul;
		if (tmp < -32768)
			tmp = -32768;
		if (tmp > 32767)
			tmp = 32767;
		data[i] = tmp;

		tmp = avg + rdiff * mul;
		if (tmp < -32768)
			tmp = -32768;
		if (tmp > 32767)
			tmp = 32767;
		data[i + 1] = tmp;
	}
	return length;
}
