/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson, 4Front Technologies
 *  Copyright (C) 2001       Gray Carper, Manu Sporny (xRhino, Inc.)
 *
 *  Daemonization by xRhino, Inc. 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "xmms.h"
#include "libxmms/util.h"

gfloat visbuffer[MAX_VISBUFFER][VIS_DATA_SIZE];
int peekvisbuffpos=0;
int pokevisbuffpos=0;

int connections[MAX_CONNECTIONS];
pthread_t answer_threads[MAX_CONNECTIONS];
gboolean answer_started[MAX_CONNECTIONS];
pthread_mutex_t answer_mutex[MAX_CONNECTIONS]={PTHREAD_MUTEX_INITIALIZER};
pthread_cond_t answer_cond[MAX_CONNECTIONS]={PTHREAD_COND_INITIALIZER};

int num_connections = -1;

pthread_mutex_t connection_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t status_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t status_cond = PTHREAD_COND_INITIALIZER;
static pthread_mutex_t pulse_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t pulse_cond = PTHREAD_COND_INITIALIZER;
static pthread_mutex_t command_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t running_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t command_cond = PTHREAD_COND_INITIALIZER;
static pthread_mutex_t play_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t play_cond = PTHREAD_COND_INITIALIZER;
static pthread_mutex_t buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t buffer_cond = PTHREAD_COND_INITIALIZER;
static gboolean running = TRUE, pulse_started = FALSE, command_started = FALSE,
   status_started = FALSE, play_started = FALSE, buffer_started = FALSE;
pthread_t status_thread, pulse_thread, command_thread, buffer_thread,
   play_thread, main_thread, output_grab_thread;
extern pthread_t playlist_get_info_thread;
int tmpvol = 0;

int csd_on = -1; //Our connected command socket descriptor
int csd = -1, ssd = -1; //Our listening socket descriptors

gfloat* get_buffered_vis_data()
{
   gfloat *data;

   PULSE_LOCK();
   
   data = visbuffer[peekvisbuffpos];
   if(peekvisbuffpos < MAX_VISBUFFER-1)
      peekvisbuffpos++;
   else
      peekvisbuffpos = 0;
   
   PULSE_UNLOCK();
   
   return data;
}

void put_buffered_vis_data(gfloat* addme)
{
   int i;
   
   PULSE_LOCK();

   for(i=0;i<VIS_DATA_SIZE;i++)
      visbuffer[pokevisbuffpos][i]=addme[i];
   
   if(pokevisbuffpos < MAX_VISBUFFER-1)
      pokevisbuffpos++;
   else
      pokevisbuffpos = 0;

   PULSE_UNLOCK();
}   

int send_burst(int sdesc, char *line_to_send)
{
   if (!send(sdesc,line_to_send, strlen(line_to_send), 0))
   {//MSG_DONTWAIT
      perror("send");
      return ERROR;
   }
   else
   {
      return 0;
   }
}

int read_burst(int newSd, char *line_to_return)
{
   int n;
   
   memset(line_to_return,0x0,MAX_MSG);
   n = recv(newSd, line_to_return, MAX_MSG, 0);
   if (n<0)
   {
      perror(" cannot receive data ");
      return ERROR;
   }
   else if (n==0)
   {
      printf("connection closed by command client\n");
      return ERROR;
   }
   else
   {
      return n;
   }
}

void lose_connection(int descriptor)
{
   int i,j;
   int temp_connections[MAX_CONNECTIONS];

   C_LOCK(); //Lock the connection list mutex

   /* Clean up the socket. */
   if(descriptor)
   {
      close(descriptor);
      shutdown(descriptor, SHUT_RDWR);
   }

   /* Remove the descriptor from our
      accounting information. */
   
   j=0;
   
   for(i=0;i<=num_connections;i++)
   {
      if(connections[i]!=descriptor)
      {
         temp_connections[j]=connections[i];
         j++;
      }
   }

   num_connections--;
   
   for(i=0;i<=num_connections;i++)
   {
      connections[i]=temp_connections[i];
   }
   C_UNLOCK(); //Unlock the connection list mutex
}

void handshake_client(int sd)
{
   /* Our document header. */
   send_xmlheader(sd);

   /* The songs in the playlist. */
   send_playlist(sd);

   /* The state of the daemon. */
   send_state(sd);
}

int find_connection(int descriptor)
{
   int i;
   
   C_LOCK();
   for(i=0;i<=num_connections;i++)
   {
      if(connections[i]==descriptor)
      {
         C_UNLOCK();
         return i;
      }
   }
   C_UNLOCK();
   return -1;
}

void halt_answer_thread(int connection)
{
   pthread_mutex_lock(&answer_mutex[connection]);
   pthread_cond_signal(&answer_cond[connection]);
   pthread_mutex_unlock(&answer_mutex[connection]);
   //pthread_cancel(answer_threads[connection]);
   //pthread_join(answer_threads[connection], NULL);
}

void track_status_request(void* discriptor)
{
   int track_me, n, connection;
   static char filler[1];

   printf("Status Tracking Thread PID: %i\n",getpid());

   /* Keep track of this status connection. */
   
   track_me = (int*)discriptor;
   connection = find_connection(track_me);
   
   if(connection != -1)
   {
      pthread_mutex_lock(&answer_mutex[connection]);
      while (!answer_started[connection] && running)
         pthread_cond_wait(&answer_cond[connection],&answer_mutex[connection]);
      pthread_mutex_unlock(&answer_mutex[connection]);
      
      while(running)
      {
         memset(filler,0x0,1);
         n = recv(track_me, filler, 1, 0);
         if(n <= 0) //The connection has been closed, so drop out
         {
            printf("connection closed by status client\n");
            break;
         }
      }
      lose_connection(track_me);
      halt_answer_thread(connection);
   }
   //gprof addition...
   //_mcleanup();
   //end gprof
   pthread_exit(0);
}

void start_answer_thread(int connection, int ssd_on)
{
   /* Spawn an answer thread. */
   pthread_create(&answer_threads[connection], NULL,
                  (void*)&track_status_request, (void*) ssd_on);
   pthread_detach(answer_threads[connection]);
   
	pthread_mutex_lock(&answer_mutex[connection]);
	answer_started[connection] = TRUE;
	pthread_cond_signal(&answer_cond[connection]);
	pthread_mutex_unlock(&answer_mutex[connection]);
}

void transmit_to_client(int sd, char *message)
{
   int i;

   C_LOCK(); //Lock the connection list mutex
   if(sd == ALL_CLIENTS)
   {
      /* Send the message to every status connection. */
      for(i=0;i<=num_connections;i++)
         send_burst(connections[i], message);
   }
   else
      send_burst(sd, message);
   C_UNLOCK(); //Unlock the connection list mutex     
}

void transmit_status_message(char *message)
{
   char temp[512];

   /* Send a status message to all the clients. */
   memset(temp,0x0,512);
   sprintf(temp, "<status message=\"%s\"/>", message);
   transmit_to_client(ALL_CLIENTS, temp);
}

void init_answer_accounting()
{
   int i;
   
   for(i=0;i<MAX_CONNECTIONS;i++)
   {
      answer_started[i] = FALSE;
   }
}

void shutdown_status_connections()
{
   int i;

   C_LOCK(); //Lock the connection list mutex

   /* Halt all the status answering threads. */
   for(i=0;i<=num_connections;i++)
      halt_answer_thread(i);
   
   /* Drop all our listening status connections. */
   for(i=0;i<=num_connections;i++)
   {
      if(connections[i])
      {
         close(connections[i]);
         shutdown(connections[i], SHUT_RDWR);
      }
   }
   num_connections = -1;

   C_UNLOCK(); //Unlock the connection list mutex
}

void answer_status_request()
{
   int cliStatLen, ssd_on;
   struct sockaddr_in cliStatAddr;

   printf("Status Thread PID: %i\n",getpid());

   pthread_mutex_lock(&status_mutex);
	while (!status_started && running)
		pthread_cond_wait(&status_cond, &status_mutex);
	pthread_mutex_unlock(&status_mutex);
   
   init_answer_accounting();
   
   while(running)
   {
      if(num_connections < MAX_CONNECTIONS-1)
      {
         cliStatLen = sizeof(cliStatAddr);
         ssd_on = accept(ssd, (struct sockaddr *) &cliStatAddr,
                         &cliStatLen);

         if(ssd_on < 0)
            continue;

         /* Send our outgoing XML header to this new client. */
         handshake_client(ssd_on);

         /* Setup connection accounting info. */
         C_LOCK(); //Lock the connection list mutex
         connections[num_connections+1]=ssd_on;
         num_connections++;
         C_UNLOCK(); //Unlock the connection list mutex

         /* Spawn a thread to track the new connection. */
         start_answer_thread(num_connections, ssd_on);
      }
   }
   pthread_mutex_lock(&status_mutex);
   status_started = FALSE;
   pthread_mutex_unlock(&status_mutex);
   //gprof addition...
   //_mcleanup();
   //end gprof
   pthread_exit(0);
}

void buffer_vis_data(void **args)
{
   int pulsecount=0;
   int mswait = args[0];
   int pulserate = args[1];
   struct timeval pretime;
   struct timeval posttime;
   long remainder;

   printf("Buffer Thread PID: %i\n",getpid());

   pthread_mutex_lock(&buffer_mutex);
	while (!buffer_started && running)
		pthread_cond_wait(&buffer_cond, &buffer_mutex);
   pthread_mutex_unlock(&buffer_mutex);

   if(mswait>=10) //Adjust for 10ms resolution
      mswait = mswait/10;
   
   /* Buffer the initial amount requested. */
   while(pulsecount < mswait)
   {
      gettimeofday(&pretime, NULL);
      put_buffered_vis_data(get_active_vis_data());
      gettimeofday(&posttime, NULL);
      remainder = posttime.tv_usec - pretime.tv_usec;
      if(remainder < 1000)
         xmms_usleep(1000-remainder);
      
      pulsecount++;
   }

   /* Allow the pulse thread to begin. */
   start_pulse_thread(pulserate);

   /* Buffer until we shutdown! */
   while(running)
   {
      gettimeofday(&pretime, NULL);
      put_buffered_vis_data(get_active_vis_data());
      gettimeofday(&posttime, NULL);
      remainder = posttime.tv_usec - pretime.tv_usec;
      xmms_usleep((1000000-remainder)/pulserate);
      //xmms_usleep(1000000/pulserate);
   }
   pthread_mutex_lock(&buffer_mutex);
   buffer_started = FALSE;
   pthread_mutex_unlock(&buffer_mutex);
   pthread_exit(0);
}

void send_pulse(void *pulserate)
{
   char temp[1024];
   gfloat *data;
   int pulse_count = 1;
   int pulse;
   struct timeval pretime;
   struct timeval posttime;
   long remainder;

   printf("Pulse Thread PID: %i\n",getpid());

   PULSE_LOCK();
	while (!pulse_started && running)
		pthread_cond_wait(&pulse_cond, &pulse_mutex);
   PULSE_UNLOCK();
   
   pulse = (int*)pulserate;
   
   /* Send a pulse of information to connected clients. */
   while(running)
   {
      gettimeofday(&pretime, NULL);
      
      /* Visualization data */
      memset(temp,0x0,1024);
      data = get_buffered_vis_data();
      sprintf(temp,"<visdata>%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,"
              "%f,%f,%f,%f,%f,%f,%f,%f,%f,%f</visdata>",
              data[0],data[1],data[2],data[3],data[4],
              data[5],data[6],data[7],data[8],data[9],
              data[10],data[11],data[12],data[13],data[14],
              data[15],data[16],data[17],data[18],data[19]);
      transmit_to_client(ALL_CLIENTS,temp);
      
      if(pulse_count == pulse/2) //Twice every second, we...
      {
         /* Report the time state */
         memset(temp,0x0,1024);
         sprintf(temp, "<seek time=\"%i\"/>", input_get_time()/1000);
         transmit_to_client(ALL_CLIENTS,temp);
         pulse_count = 1;
      }
      else
         pulse_count++;
      gettimeofday(&posttime, NULL);
      remainder = posttime.tv_usec - pretime.tv_usec;
      xmms_usleep((1000000-remainder)/pulse);
      //xmms_usleep(1000000/pulse);
   }
   PULSE_LOCK();
   pulse_started = FALSE;
   PULSE_UNLOCK();
   pthread_exit(0);
}

void answer_command_request()
{
   int cliCommLen;
   int done = 0;
   char line[MAX_MSG];
   struct sockaddr_in cliCommAddr;

   printf("Command Thread PID: %i\n",getpid());
   
   pthread_mutex_lock(&command_mutex);
	while (!command_started && running)
		pthread_cond_wait(&command_cond, &command_mutex);
	pthread_mutex_unlock(&command_mutex);
   
   /* Handle incoming command connections to the daemon. */
   
   while(running)
   {
      cliCommLen = sizeof(cliCommAddr);
      csd_on = accept(csd, (struct sockaddr *) &cliCommAddr,
                      &cliCommLen);
      if(csd_on<0)
         continue;
      
      /* XML Initialization */
      xml_comm_init();
      memset(line,0x0,MAX_MSG);

      /* Receive segments and pass them to the XML parser. */
      while(read_burst(csd_on,line) != ERROR && running)
      {
         if(strstr(line, CLOSING_TAG) != NULL)
            done = 1;
         else
            done = 0;
         
         if (!xml_parse_this(line, done) || done) //Clean up on error
         {                                        //or end of document.
            done = 1; //If this is an error, we need to force a 'done'
                      //state.
            close(csd_on);
            shutdown(csd_on, SHUT_RDWR);
            csd_on = -1;
         }
         memset(line,0x0,MAX_MSG);
      }
      
      /* Cleanup our XML parser and connection (as long as we didn't
         do it earlier). */
      xml_comm_close();
      if(!done)
      {
         close(csd_on);
         shutdown(csd_on, SHUT_RDWR);
         csd_on = -1;
      }
   }
   pthread_mutex_lock(&command_mutex);
   command_started = FALSE;
   pthread_mutex_unlock(&command_mutex);
   pthread_exit(0);
}

gint idle_func(void *pulserate)
{
	gint time, t, length;
	gchar stime_prefix;
	static gboolean waiting = FALSE;
	static gint count = 0;
   static GTimer *pause_timer = NULL;
   int pulse;
   struct timeval pretime;
   struct timeval posttime;
   long remainder;

   printf("Play Thread PID: %i\n",getpid());

   pulse = (int*)pulserate;

   /* Keep track of the play status (among other things). */

   pthread_mutex_lock(&play_mutex);
	while (!play_started && running)
		pthread_cond_wait(&play_cond, &play_mutex);
	pthread_mutex_unlock(&play_mutex);

   while(running)
   {
      gettimeofday(&pretime, NULL);
      
      if(get_input_playing())
      {
         time = input_get_time();
         if(time == -1)
         {
            if(cfg.pause_between_songs)
            {
               gint timeleft;
               if(!waiting)
               {
                  if(!pause_timer)
                     pause_timer = g_timer_new();
                  else
                     g_timer_reset(pause_timer);
                  waiting = TRUE;
               }
               timeleft = cfg.pause_between_songs_time -
                  (gint)g_timer_elapsed(pause_timer, NULL);
            }

            if(!cfg.pause_between_songs ||
               g_timer_elapsed(pause_timer,NULL) >=
               cfg.pause_between_songs_time)
            {
               playlist_eof_reached();
               waiting = FALSE;
            }
         }
         else
         {
            length = playlist_get_current_length();
            input_update_vis(time);
            
            if(cfg.timer_mode == TIMER_REMAINING)
            {
               if (length != -1)
               {
                  t = length - time;
                  stime_prefix = '-';
               }
               else
               {
                  t = time;
                  stime_prefix = ' ';
               }
            }
            else
            {
               t = time;
               stime_prefix = ' ';
            }
            t /= 1000;

            /*
             * Show the time in the format HH:MM when we
             * have more than 100 minutes.
             */
            if(t >= 100 * 60)
               t /= 60;
            
            time /= 1000;
            length /= 1000;
         }

         if(time != -1)
            waiting = FALSE;
      }
      check_ctrlsocket();

      if(!count)
      {
         read_volume(VOLSET_UPDATE);
         count = 10;
      }
      else
         count--;
      
      gettimeofday(&posttime, NULL);
      remainder = posttime.tv_usec - pretime.tv_usec;
      xmms_usleep((1000000-remainder)/pulse);
      //xmms_usleep(1000000/pulse);
   }
   pthread_mutex_lock(&play_mutex);
   play_started = FALSE;
   pthread_mutex_unlock(&play_mutex);
   pthread_exit(0);
}

/* Socket Opening and Closing */
int open_command_socket()
{
   struct sockaddr_in servCommAddr;
   
   /* Create the listening command socket */
   printf("Opening command socket..");
   fflush(0);
   
   while(csd<0)
   {
      csd = socket(AF_INET, SOCK_STREAM, 0);
      sleep(1);
      printf(".");
      fflush(0);
   }
   printf("done, assigned to %i.\n",csd);
   fflush(0);

   /* Bind the server command port */
   servCommAddr.sin_family = AF_INET;
   servCommAddr.sin_addr.s_addr = htonl(INADDR_ANY);
   servCommAddr.sin_port = htons(SERVER_COMMAND_PORT);

   printf("Binding %i to command port %i...",csd,SERVER_COMMAND_PORT);
   fflush(0);
   while(bind(csd, (struct sockaddr *) &servCommAddr, sizeof(servCommAddr))<0)
   {
      sleep(1);
      printf(".");
      fflush(0);
   }
   printf("done\n");
   fflush(0);

   /* Listen on the command port... */
   listen(csd,0);

   return 0;
}

int open_status_socket()
{
   struct sockaddr_in servStatAddr;
   
   /* Create the listening socket */
   printf("Opening status socket..");
   fflush(0);
   while(ssd<0)
   {
      ssd = socket(AF_INET, SOCK_STREAM, 0);
      sleep(1);
      printf(".");
      fflush(0);
   }
   printf("done, assigned to %i.\n",ssd);
   fflush(0);

   /* Bind the server status port */
   servStatAddr.sin_family = AF_INET;
   servStatAddr.sin_addr.s_addr = htonl(INADDR_ANY);
   servStatAddr.sin_port = htons(SERVER_STATUS_PORT);

   printf("Binding %i to status port %i...",ssd,SERVER_STATUS_PORT);
   fflush(0);
   while(bind(ssd, (struct sockaddr *) &servStatAddr, sizeof(servStatAddr))<0)
   {
      sleep(1);
      printf(".");
      fflush(0);
   }
   printf("done\n");
   fflush(0);

   /* Listen on the status port... */
   listen(ssd,0);

   return 0;
}

void close_command_socket()
{
   close(csd_on);
   shutdown(csd_on, SHUT_RDWR);
   csd_on=-1;
   close(csd);
   shutdown(csd, SHUT_RDWR);
	csd = -1;
   printf("Command socket closed.\n");
}

void close_status_socket()
{
   shutdown_status_connections(); //Kill our status connections.
   close(ssd);
   shutdown(ssd, SHUT_RDWR);
   ssd = -1;
   printf("Status socket closed.\n");
}

void start_command_thread()
{
   /* Spawn the command socket handling thread. */
   open_command_socket();
   
   pthread_mutex_lock(&command_mutex);
   if(!command_started)
   {
      command_started = TRUE;
      pthread_cond_signal(&command_cond);
      pthread_mutex_unlock(&command_mutex);
      
      pthread_create(&command_thread, NULL, (void*)&answer_command_request,
                     NULL);
   }
   else
      pthread_mutex_unlock(&command_mutex);
   
}

void start_status_thread()
{
   /* Spawn the status socket handling thread. */
   open_status_socket();
   
   pthread_mutex_lock(&status_mutex);
   if(!status_started)
   {
      status_started = TRUE;
      pthread_cond_signal(&status_cond);
      pthread_mutex_unlock(&status_mutex);
      pthread_create(&status_thread, NULL, (void*)&answer_status_request,
                     NULL);
   }
   else
      pthread_mutex_unlock(&status_mutex);
}

void start_pulse_thread(int pulserate)
{
   /* Spawn the information pulse thread. */
   PULSE_LOCK();
   if(!pulse_started)
   {
      pulse_started = TRUE;
      pthread_cond_signal(&pulse_cond);
      PULSE_UNLOCK();
      pthread_create(&pulse_thread, NULL, (void*)&send_pulse,
                     (void*) pulserate);
   }
   else
      PULSE_UNLOCK();
}

void start_buffer_thread(int *args)
{
   /* Spawn the visualization data buffering thread. */
   pthread_mutex_lock(&buffer_mutex);
   if(!buffer_started)
   {
      buffer_started = TRUE;
      pthread_cond_signal(&buffer_cond);
      pthread_mutex_unlock(&buffer_mutex);
      pthread_create(&buffer_thread, NULL, (void*)&buffer_vis_data,
                     (void*) args);
   }
   else
      pthread_mutex_unlock(&buffer_mutex);
}

void start_play_thread(int pulserate)
{
   /* Spawn a thread to keep track of the play status. */
   pthread_mutex_lock(&play_mutex);
   if(!play_started)
   {
      play_started = TRUE;
      pthread_cond_signal(&play_cond);
      pthread_mutex_unlock(&play_mutex);
      pthread_create(&play_thread, NULL, (void*)&idle_func, (void*) pulserate);
   }
   else
      pthread_mutex_unlock(&play_mutex);
}

void halt_play_thread(void)
{
   int block = 1;
   
   while(block) 
   {
      pthread_mutex_lock(&play_mutex);
      if(!play_started)
         block = 0;
      else
         stop_running();
      pthread_mutex_unlock(&play_mutex);
   }
   pthread_mutex_lock(&play_mutex);
   pthread_cond_signal(&play_cond);
   pthread_mutex_unlock(&play_mutex);
   pthread_join(play_thread, NULL);
   printf("Play thread killed.\n");
}

void halt_pulse_thread(void)
{
   int block = 1;
   
   while(block)
   {
      PULSE_LOCK();
      if(!pulse_started)
         block = 0;
      else
         stop_running();
      PULSE_UNLOCK();
   }
   PULSE_LOCK();
   pthread_cond_signal(&pulse_cond);
   PULSE_UNLOCK();
   pthread_join(pulse_thread, NULL);
   printf("Pulse thread killed.\n");
}

void halt_buffer_thread(void)
{
   int block = 1;
   
   halt_pulse_thread(); //Kill our little pulse thread first
   
   while(block)
   {
      pthread_mutex_lock(&buffer_mutex);
      if(!buffer_started)
         block = 0;
      else
         stop_running();
      pthread_mutex_unlock(&buffer_mutex);
   }
   pthread_mutex_lock(&buffer_mutex);
   pthread_cond_signal(&buffer_cond);
   pthread_mutex_unlock(&buffer_mutex);
   pthread_join(buffer_thread, NULL);
   printf("Buffer thread killed.\n");
}

void halt_status_thread(void)
{
   int block = 1;
   
   while(block > 0 && block<BLOCK_TIMEOUT)
   {
      pthread_mutex_lock(&status_mutex);
      if(!status_started)
         block = 0;
      else
      {
         stop_running();
         block++;
      }
      pthread_mutex_unlock(&status_mutex);
      
   }
   pthread_mutex_lock(&status_mutex);
   pthread_cond_signal(&status_cond);
   pthread_mutex_unlock(&status_mutex);
   pthread_join(status_thread, NULL);
   printf("Status thread killed.\n");
}

void halt_command_thread(void)
{
   int block = 1;

   while(block > 0 && block<BLOCK_TIMEOUT)
   {
      pthread_mutex_lock(&command_mutex);
      if(!command_started)
         block = 0;
      else
      {
         stop_running();
         block++;
      }
      pthread_mutex_unlock(&command_mutex);
   }
   pthread_mutex_lock(&command_mutex);
   pthread_cond_signal(&command_cond);
   pthread_mutex_unlock(&command_mutex);
   pthread_join(command_thread, NULL);
   printf("Command thread killed.\n");
}

void stop_running()
{
   pthread_mutex_lock(&running_mutex);
   running = FALSE;
   pthread_mutex_unlock(&running_mutex);
}

void set_main_thread(pthread_t to_this_thread)
{
   main_thread = to_this_thread;
}

pthread_t get_main_thread()
{
   return main_thread;
}

char* which_thread()
{
   pthread_t curr_thread = pthread_self();

   if(pthread_equal(curr_thread,status_thread))
      return "status";
   else if(pthread_equal(curr_thread,pulse_thread))
      return "pulse";
   else if(pthread_equal(curr_thread,command_thread))
      return "command";
   else if(pthread_equal(curr_thread,buffer_thread))
      return "buffer";
   else if(pthread_equal(curr_thread,play_thread))
      return "play";
   else if(pthread_equal(curr_thread,main_thread))
      return "main";
   else if(pthread_equal(curr_thread,playlist_get_info_thread))
      return "xmms playlist";
   else
      return "unknown (xmms?)";
}

void drop_sockets()
{
   close_command_socket();
   close_status_socket();
}
