/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson, 4Front Technologies
 *  Copyright (C) 2001       Gray Carper, Manu Sporny (xRhino, Inc.)
 *
 *  Daemonization by xRhino, Inc. 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DAEMON_H_
#define _DAEMON_H_

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>

typedef struct
{
   char title[256];
	char artist[31];
	char album[31];
	char year[5];
	char comment[31];
   char genre[256];
   char station[256];
   char channels[15];
   char error_protection[5];
	int bitrate;
	int sampling_frequency;
   int frames;
	char copyright[5];
	char original[5];
	char emphasis[10];
   int filesize;
} FileInfo;

#define SUCCESS 0
#define ERROR   1
#define SERVER_STATUS_PORT 9001
#define SERVER_COMMAND_PORT 9000
#define MAX_MSG 2048
#define BLOCK_TIMEOUT 50000
#define MAX_CONNECTIONS 50
#define MAX_VISBUFFER 1000
#define VIS_DATA_SIZE 20
#define ALL_CLIENTS -1
#define C_LOCK()    pthread_mutex_lock(&connection_mutex)
#define C_UNLOCK()  pthread_mutex_unlock(&connection_mutex)
#define PULSE_LOCK()    pthread_mutex_lock(&pulse_mutex)
#define PULSE_UNLOCK()  pthread_mutex_unlock(&pulse_mutex)

int send_burst(int sdesc, char *line_to_send);
int read_burst(int newSd, char *line_to_return);
void transmit_to_client(int sd, char *message);
void transmit_status_message(char *message);
void shutdown_status_connections(void);
void start_status_thread(void);
void start_pulse_thread(int pulserate);
void start_buffer_thread(int *args);
void start_command_thread();
void start_play_thread(int pulserate);
void halt_play_thread(void);
void halt_status_thread(void);
void halt_pulse_thread(void);
void halt_buffer_thread(void);
void halt_command_thread(void);
void stop_running(void);
void set_main_thread(pthread_t to_this_thread);
pthread_t get_main_thread(void);
void drop_sockets(void);
char* which_thread(void);
void grab_output_device(void);

#endif //_DAEMON_H_
