/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "xmms.h"
#include "libxmms/configfile.h"
#include "libxmms/xmmsctrl.h"

GList *disabled_iplugins = NULL;
gint bitrate = 0, frequency = 0, numchannels = 0;
Config cfg;
gboolean pposition_broken = FALSE;
extern gchar *plugin_dir_list[];
int mswait = 1; //How long to we prebuffer the vis data? (Default)
int pulserate = 60; //How many times a second do we send a status
                    //pulse? (Default)
SVis *mainwin_svis;
Vis *active_vis;

static void make_xmms_dir(void)
{
	gchar *filename;

	//filename = g_strconcat(get_ps2_mc(), "/", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/", NULL);
	mkdir(filename, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
	g_free(filename);
	//filename = g_strconcat(get_ps2_mc(), "/", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/", NULL);
	mkdir(filename, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
	g_free(filename);
   umount_ps2_mc();
}

static void read_config(void)
{
	ConfigFile *cfgfile;
	gchar *filename;
	gint i, length;

	memset(&cfg, 0, sizeof (Config));
	cfg.autoscroll = TRUE;
	cfg.player_x = 20;
	cfg.player_y = 20;
	cfg.always_on_top = FALSE;
	cfg.sticky = FALSE;
	cfg.always_show_cb = TRUE;
	cfg.convert_underscore = TRUE;
	cfg.convert_twenty = TRUE;
	cfg.show_numbers_in_pl = TRUE;
	cfg.snap_windows = TRUE;
	cfg.save_window_position = TRUE;
	cfg.dim_titlebar = TRUE;
	cfg.open_rev_order = FALSE;
	cfg.get_info_on_load = FALSE;
	cfg.get_info_on_demand = TRUE;
	cfg.eq_doublesize_linked = TRUE;
	cfg.player_visible = TRUE;
	cfg.no_playlist_advance = FALSE;
	cfg.smooth_title_scroll = TRUE;
	cfg.random_skin_on_play = FALSE;
	cfg.mainwin_use_xfont = FALSE;

	cfg.playlist_x = 295;
	cfg.playlist_y = 20;
	cfg.playlist_width = 300;
	cfg.playlist_height = 232;

	cfg.filesel_path = NULL;
	cfg.playlist_path = NULL;

	cfg.equalizer_x = 20;
	cfg.equalizer_y = 136;

	cfg.snap_distance = 10;
	cfg.pause_between_songs_time = 2;

	cfg.disabled_iplugins = NULL;
	cfg.enabled_gplugins = NULL;
	cfg.mouse_change = 8;

   //filename = g_strconcat(get_ps2_mc(), "/config", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
   
	cfgfile = xmms_cfg_open_file(filename);
	if (cfgfile)
	{
		xmms_cfg_read_boolean(cfgfile, "xmms", "allow_multiple_instances", &cfg.allow_multiple_instances);
		xmms_cfg_read_boolean(cfgfile, "xmms", "use_realtime", &cfg.use_realtime);
		xmms_cfg_read_boolean(cfgfile, "xmms", "always_show_cb", &cfg.always_show_cb);
		xmms_cfg_read_boolean(cfgfile, "xmms", "convert_underscore", &cfg.convert_underscore);
		xmms_cfg_read_boolean(cfgfile, "xmms", "convert_%20", &cfg.convert_twenty);
		xmms_cfg_read_boolean(cfgfile, "xmms", "show_numbers_in_pl", &cfg.show_numbers_in_pl);
		xmms_cfg_read_boolean(cfgfile, "xmms", "snap_windows", &cfg.snap_windows);
		xmms_cfg_read_boolean(cfgfile, "xmms", "save_window_positions", &cfg.save_window_position);
		xmms_cfg_read_boolean(cfgfile, "xmms", "dim_titlebar", &cfg.dim_titlebar);
		xmms_cfg_read_boolean(cfgfile, "xmms", "save_playlist_position", &cfg.save_playlist_position);
		xmms_cfg_read_boolean(cfgfile, "xmms", "open_rev_order", &cfg.open_rev_order);
		xmms_cfg_read_boolean(cfgfile, "xmms", "get_info_on_load", &cfg.get_info_on_load);
		xmms_cfg_read_boolean(cfgfile, "xmms", "get_info_on_demand", &cfg.get_info_on_demand);
		xmms_cfg_read_boolean(cfgfile, "xmms", "eq_doublesize_linked", &cfg.eq_doublesize_linked);
		xmms_cfg_read_boolean(cfgfile, "xmms", "no_playlist_advance", &cfg.no_playlist_advance);
		xmms_cfg_read_boolean(cfgfile, "xmms", "sort_jump_to_file", &cfg.sort_jump_to_file);
		xmms_cfg_read_boolean(cfgfile, "xmms", "smooth_title_scroll", &cfg.smooth_title_scroll);
		xmms_cfg_read_boolean(cfgfile, "xmms", "use_backslash_as_dir_delimiter", &cfg.use_backslash_as_dir_delimiter);
		xmms_cfg_read_boolean(cfgfile, "xmms", "enable_dga", &cfg.enable_dga);
		xmms_cfg_read_int(cfgfile, "xmms", "player_x", &cfg.player_x);
		xmms_cfg_read_int(cfgfile, "xmms", "player_y", &cfg.player_y);
		xmms_cfg_read_boolean(cfgfile, "xmms", "player_shaded", &cfg.player_shaded);
		xmms_cfg_read_boolean(cfgfile, "xmms", "shuffle", &cfg.shuffle);
		xmms_cfg_read_boolean(cfgfile, "xmms", "repeat", &cfg.repeat);
		xmms_cfg_read_boolean(cfgfile, "xmms", "doublesize", &cfg.doublesize);
		xmms_cfg_read_boolean(cfgfile, "xmms", "autoscroll_songname", &cfg.autoscroll);
		xmms_cfg_read_int(cfgfile, "xmms", "timer_mode", &cfg.timer_mode);
		xmms_cfg_read_int(cfgfile, "xmms", "vis_type", &cfg.vis_type);
		xmms_cfg_read_int(cfgfile, "xmms", "analyzer_mode", &cfg.analyzer_mode);
		xmms_cfg_read_int(cfgfile, "xmms", "analyzer_type", &cfg.analyzer_type);
		xmms_cfg_read_boolean(cfgfile, "xmms", "analyzer_peaks", &cfg.analyzer_peaks);
		xmms_cfg_read_int(cfgfile, "xmms", "scope_mode", &cfg.scope_mode);
		xmms_cfg_read_int(cfgfile, "xmms", "vu_mode", &cfg.vu_mode);
		xmms_cfg_read_int(cfgfile, "xmms", "vis_refresh_rate", &cfg.vis_refresh);
		xmms_cfg_read_int(cfgfile, "xmms", "analyzer_falloff", &cfg.analyzer_falloff);
		xmms_cfg_read_int(cfgfile, "xmms", "peaks_falloff", &cfg.peaks_falloff);
		xmms_cfg_read_int(cfgfile, "xmms", "playlist_x", &cfg.playlist_x);
		xmms_cfg_read_int(cfgfile, "xmms", "playlist_y", &cfg.playlist_y);
		xmms_cfg_read_int(cfgfile, "xmms", "playlist_width", &cfg.playlist_width);
		xmms_cfg_read_int(cfgfile, "xmms", "playlist_height", &cfg.playlist_height);
		xmms_cfg_read_boolean(cfgfile, "xmms", "playlist_shaded", &cfg.playlist_shaded);
		xmms_cfg_read_boolean(cfgfile, "xmms", "playlist_visible", &cfg.playlist_visible);
		xmms_cfg_read_string(cfgfile, "xmms", "playlist_font", &cfg.playlist_font);
		xmms_cfg_read_boolean(cfgfile, "xmms", "use_fontsets", &cfg.use_fontsets);
		xmms_cfg_read_int(cfgfile, "xmms", "playlist_position", &cfg.playlist_position);
		xmms_cfg_read_int(cfgfile, "xmms", "equalizer_x", &cfg.equalizer_x);
		xmms_cfg_read_int(cfgfile, "xmms", "equalizer_y", &cfg.equalizer_y);
		xmms_cfg_read_int(cfgfile, "xmms", "snap_distance", &cfg.snap_distance);
		xmms_cfg_read_boolean(cfgfile, "xmms", "equalizer_visible", &cfg.equalizer_visible);
		xmms_cfg_read_boolean(cfgfile, "xmms", "equalizer_active", &cfg.equalizer_active);
		xmms_cfg_read_boolean(cfgfile, "xmms", "equalizer_shaded", &cfg.equalizer_shaded);
		xmms_cfg_read_boolean(cfgfile, "xmms", "equalizer_autoload", &cfg.equalizer_autoload);
		xmms_cfg_read_boolean(cfgfile, "xmms", "easy_move", &cfg.easy_move);
		xmms_cfg_read_float(cfgfile, "xmms", "equalizer_preamp", &cfg.equalizer_preamp);
		for (i = 0; i < 10; i++)
		{
			gchar eqtext[18];
			
			sprintf(eqtext, "equalizer_band%d", i);
			xmms_cfg_read_float(cfgfile, "xmms", eqtext, &cfg.equalizer_bands[i]);
		}
  		xmms_cfg_read_string(cfgfile, "xmms", "eqpreset_default_file", &cfg.eqpreset_default_file);
		xmms_cfg_read_string(cfgfile, "xmms", "eqpreset_extension", &cfg.eqpreset_extension);
		xmms_cfg_read_string(cfgfile, "xmms", "skin", &cfg.skin);
		xmms_cfg_read_string(cfgfile, "xmms", "output_plugin", &cfg.outputplugin);
		xmms_cfg_read_string(cfgfile, "xmms", "effect_plugin", &cfg.effectplugin);
		xmms_cfg_read_string(cfgfile, "xmms", "enabled_gplugins", &cfg.enabled_gplugins);
		xmms_cfg_read_string(cfgfile, "xmms", "enabled_vplugins", &cfg.enabled_vplugins);
		xmms_cfg_read_string(cfgfile, "xmms", "filesel_path", &cfg.filesel_path);
		xmms_cfg_read_string(cfgfile, "xmms", "playlist_path", &cfg.playlist_path);
		xmms_cfg_read_string(cfgfile, "xmms", "disabled_iplugins", &cfg.disabled_iplugins);
		xmms_cfg_read_boolean(cfgfile, "xmms", "use_eplugins", &cfg.use_eplugins);
		xmms_cfg_read_boolean(cfgfile, "xmms", "always_on_top", &cfg.always_on_top);
		xmms_cfg_read_boolean(cfgfile, "xmms", "sticky", &cfg.sticky);
		xmms_cfg_read_boolean(cfgfile, "xmms", "random_skin_on_play", &cfg.random_skin_on_play);
		xmms_cfg_read_boolean(cfgfile, "xmms", "pause_between_songs", &cfg.pause_between_songs);
		xmms_cfg_read_int(cfgfile, "xmms", "pause_between_songs_time", &cfg.pause_between_songs_time);
		xmms_cfg_read_int(cfgfile, "xmms", "mouse_wheel_change", &cfg.mouse_change);
		if (xmms_cfg_read_int(cfgfile, "xmms", "url_history_length", &length))
		{
			for(i = 1; i <= length; i++)
			{
				gchar str[19], *temp;
				
				sprintf(str, "url_history%d", i);
				if (xmms_cfg_read_string(cfgfile, "xmms", str, &temp))
					cfg.url_history = g_list_append(cfg.url_history, temp);
			}
		}
		
		xmms_cfg_free(cfgfile);
	}

	if (cfg.playlist_font == NULL)
		cfg.playlist_font = g_strdup("-adobe-helvetica-bold-r-*-*-10-*");
	if (cfg.mainwin_font == NULL)
		cfg.mainwin_font = g_strdup("-adobe-helvetica-medium-r-*-*-8-*");
	if (cfg.outputplugin == NULL)
	{
#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
		cfg.outputplugin = g_strdup_printf("%s/%s/libOSS.so", PLUGIN_DIR, plugin_dir_list[0]);
#elif defined(sun)
		cfg.outputplugin = g_strdup_printf("%s/%s/libSolaris.so", PLUGIN_DIR, plugin_dir_list[0]);
#else
		/*
		 * FIXME: This implisitly means the output plugin that is first
		 * in the alphabet will be used (usually the disk writer plugin)
		 */
		cfg.outputplugin = g_strdup("");
#endif
	}
	if (cfg.effectplugin == NULL)
		cfg.effectplugin = g_strdup("");
	if (cfg.eqpreset_default_file == NULL)
		cfg.eqpreset_default_file = g_strdup("dir_default.preset");
	if (cfg.eqpreset_extension == NULL)
		cfg.eqpreset_extension = g_strdup("preset");

   /* Forced For xRhino PS2 Version: */
   cfg.outputplugin = g_strdup_printf("%s/%s/libesdout.so", PLUGIN_DIR,
                                      plugin_dir_list[0]);
   //cfg.outputplugin = g_strdup_printf("%s/%s/libOSS.so", PLUGIN_DIR,
   //                                   plugin_dir_list[0]);
   
	g_free(filename);
   umount_ps2_mc();
}

void save_config(void)
{
	GList *d_iplist, *node;
	gchar *temp;
	gchar *filename, *str;
	gint i;
	ConfigFile *cfgfile;

	if (cfg.disabled_iplugins)
		g_free(cfg.disabled_iplugins);
	if (disabled_iplugins && (g_list_length(disabled_iplugins) > 0))
	{
		d_iplist = disabled_iplugins;
		cfg.disabled_iplugins = g_strdup(g_basename(((InputPlugin *) d_iplist->data)->filename));
		d_iplist = d_iplist->next;
		while (d_iplist != NULL)
		{
			temp = cfg.disabled_iplugins;
			cfg.disabled_iplugins = g_strconcat(temp, ",", g_basename(((InputPlugin *) d_iplist->data)->filename), NULL);
			g_free(temp);
			d_iplist = d_iplist->next;
		}
	}
	else
		cfg.disabled_iplugins = g_strdup("");
	//filename = g_strconcat(get_ps2_mc(), "/config", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfgfile = xmms_cfg_open_file(filename);
	if (!cfgfile)
		cfgfile = xmms_cfg_new();
	xmms_cfg_write_boolean(cfgfile, "xmms", "allow_multiple_instances", cfg.allow_multiple_instances);
	xmms_cfg_write_boolean(cfgfile, "xmms", "use_realtime", cfg.use_realtime);
	xmms_cfg_write_boolean(cfgfile, "xmms", "always_show_cb", cfg.always_show_cb);
	xmms_cfg_write_boolean(cfgfile, "xmms", "convert_underscore", cfg.convert_underscore);
	xmms_cfg_write_boolean(cfgfile, "xmms", "convert_%20", cfg.convert_twenty);
	xmms_cfg_write_boolean(cfgfile, "xmms", "show_numbers_in_pl", cfg.show_numbers_in_pl);
	xmms_cfg_write_boolean(cfgfile, "xmms", "snap_windows", cfg.snap_windows);
	xmms_cfg_write_boolean(cfgfile, "xmms", "save_window_positions", cfg.save_window_position);
	xmms_cfg_write_boolean(cfgfile, "xmms", "dim_titlebar", cfg.dim_titlebar);
	xmms_cfg_write_boolean(cfgfile, "xmms", "save_playlist_position", cfg.save_playlist_position);
	xmms_cfg_write_boolean(cfgfile, "xmms", "open_rev_order", cfg.open_rev_order);
	xmms_cfg_write_boolean(cfgfile, "xmms", "get_info_on_load", cfg.get_info_on_load);
	xmms_cfg_write_boolean(cfgfile, "xmms", "get_info_on_demand", cfg.get_info_on_demand);
	xmms_cfg_write_boolean(cfgfile, "xmms", "eq_doublesize_linked", cfg.eq_doublesize_linked);
	xmms_cfg_write_boolean(cfgfile, "xmms", "no_playlist_advance", cfg.no_playlist_advance);
	xmms_cfg_write_boolean(cfgfile, "xmms", "sort_jump_to_file", cfg.sort_jump_to_file);
	xmms_cfg_write_boolean(cfgfile, "xmms", "smooth_title_scroll", cfg.smooth_title_scroll);
	xmms_cfg_write_boolean(cfgfile, "xmms", "use_backslash_as_dir_delimiter", cfg.use_backslash_as_dir_delimiter);
	xmms_cfg_write_boolean(cfgfile, "xmms", "enable_dga", cfg.enable_dga);
	xmms_cfg_write_boolean(cfgfile, "xmms", "doublesize", cfg.doublesize);
	xmms_cfg_write_boolean(cfgfile, "xmms", "autoscroll_songname", cfg.autoscroll);
	xmms_cfg_write_int(cfgfile, "xmms", "timer_mode", cfg.timer_mode);
	xmms_cfg_write_int(cfgfile, "xmms", "vis_type", cfg.vis_type);
	xmms_cfg_write_int(cfgfile, "xmms", "analyzer_mode", cfg.analyzer_mode);
	xmms_cfg_write_int(cfgfile, "xmms", "analyzer_type", cfg.analyzer_type);
	xmms_cfg_write_boolean(cfgfile, "xmms", "analyzer_peaks", cfg.analyzer_peaks);
	xmms_cfg_write_int(cfgfile, "xmms", "scope_mode", cfg.scope_mode);
	xmms_cfg_write_int(cfgfile, "xmms", "vu_mode", cfg.vu_mode);
	xmms_cfg_write_int(cfgfile, "xmms", "vis_refresh_rate", cfg.vis_refresh);
	xmms_cfg_write_int(cfgfile, "xmms", "analyzer_falloff", cfg.analyzer_falloff);
	xmms_cfg_write_int(cfgfile, "xmms", "peaks_falloff", cfg.peaks_falloff);
	xmms_cfg_write_int(cfgfile, "xmms", "playlist_x", cfg.playlist_x);
	xmms_cfg_write_int(cfgfile, "xmms", "playlist_y", cfg.playlist_y);
	xmms_cfg_write_int(cfgfile, "xmms", "playlist_width", cfg.playlist_width);
	xmms_cfg_write_int(cfgfile, "xmms", "playlist_height", cfg.playlist_height);
	xmms_cfg_write_boolean(cfgfile, "xmms", "playlist_shaded", cfg.playlist_shaded);
	xmms_cfg_write_boolean(cfgfile, "xmms", "playlist_visible", cfg.playlist_visible);
	xmms_cfg_write_string(cfgfile, "xmms", "playlist_font", cfg.playlist_font);
	xmms_cfg_write_boolean(cfgfile, "xmms", "use_fontsets", cfg.use_fontsets);
	xmms_cfg_write_int(cfgfile, "xmms", "playlist_position", get_playlist_position());
	xmms_cfg_write_int(cfgfile, "xmms", "equalizer_x", cfg.equalizer_x);
	xmms_cfg_write_int(cfgfile, "xmms", "equalizer_y", cfg.equalizer_y);
	xmms_cfg_write_int(cfgfile, "xmms", "snap_distance", cfg.snap_distance);
	xmms_cfg_write_boolean(cfgfile, "xmms", "equalizer_visible", cfg.equalizer_visible);
	xmms_cfg_write_boolean(cfgfile, "xmms", "equalizer_shaded", cfg.equalizer_shaded);
	xmms_cfg_write_boolean(cfgfile, "xmms", "equalizer_active", cfg.equalizer_active);
	xmms_cfg_write_boolean(cfgfile, "xmms", "equalizer_autoload", cfg.equalizer_autoload);
	xmms_cfg_write_boolean(cfgfile, "xmms", "easy_move", cfg.easy_move);
	xmms_cfg_write_boolean(cfgfile, "xmms", "use_eplugins", cfg.use_eplugins);
	xmms_cfg_write_boolean(cfgfile, "xmms", "always_on_top", cfg.always_on_top);
	xmms_cfg_write_boolean(cfgfile, "xmms", "sticky", cfg.sticky);
	xmms_cfg_write_float(cfgfile, "xmms", "equalizer_preamp", cfg.equalizer_preamp);
	xmms_cfg_write_boolean(cfgfile, "xmms", "random_skin_on_play", cfg.random_skin_on_play);
	xmms_cfg_write_boolean(cfgfile, "xmms", "pause_between_songs", cfg.pause_between_songs);
	xmms_cfg_write_int(cfgfile, "xmms", "pause_between_songs_time", cfg.pause_between_songs_time);
	xmms_cfg_write_int(cfgfile, "xmms", "mouse_wheel_change", cfg.mouse_change);
 	xmms_cfg_write_string(cfgfile, "xmms", "eqpreset_default_file", cfg.eqpreset_default_file);
 	xmms_cfg_write_string(cfgfile, "xmms", "eqpreset_extension", cfg.eqpreset_extension);
	for (i = 0; i < 10; i++)
	{
		str = g_strdup_printf("equalizer_band%d", i);
		xmms_cfg_write_float(cfgfile, "xmms", str, cfg.equalizer_bands[i]);
		g_free(str);
	}
   xmms_cfg_remove_key(cfgfile, "xmms", "skin");
	if (get_current_output_plugin())
		xmms_cfg_write_string(cfgfile, "xmms", "output_plugin", get_current_output_plugin()->filename);
	else
		xmms_cfg_remove_key(cfgfile, "xmms", "output_plugin");
	if (get_current_effect_plugin())
		xmms_cfg_write_string(cfgfile, "xmms", "effect_plugin", get_current_effect_plugin()->filename);
	else
		xmms_cfg_remove_key(cfgfile, "xmms", "effect_plugin");

	str = general_stringify_enabled_list();
	if (str)
	{
		xmms_cfg_write_string(cfgfile, "xmms", "enabled_gplugins", str);
		g_free(str);
	}
	else
		xmms_cfg_remove_key(cfgfile, "xmms", "enabled_gplugins");
   xmms_cfg_remove_key(cfgfile, "xmms", "enabled_vplugins");
	xmms_cfg_write_string(cfgfile, "xmms", "disabled_iplugins", cfg.disabled_iplugins);
	if (cfg.filesel_path)
		xmms_cfg_write_string(cfgfile, "xmms", "filesel_path", cfg.filesel_path);
	if (cfg.playlist_path)
		xmms_cfg_write_string(cfgfile, "xmms", "playlist_path", cfg.playlist_path);
	xmms_cfg_write_int(cfgfile, "xmms", "url_history_length", g_list_length(cfg.url_history));
	for (node = cfg.url_history, i = 1; node; node = g_list_next(node), i++)
	{
		str = g_strdup_printf("url_history%d", i);
		xmms_cfg_write_string(cfgfile, "xmms", str, node->data);
		g_free(str);
	}
	
	xmms_cfg_write_file(cfgfile, filename);
	xmms_cfg_free(cfgfile);

	g_free(filename);
	//filename = g_strconcat(get_ps2_mc(), "/xmms.m3u", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/xmms.m3u", NULL);
	playlist_save(filename);
	g_free(filename);
   umount_ps2_mc();
}

struct option long_options[] =
{
	{"help", 0, 0, 0},
	{"buffer-vis-data", 1, 0, 0},
   {"pulse-rate", 1, 0, 0},
	{"version", 0, 0, 0},
	{0, 0, 0, 0}
};

void display_usage(void)
{
	fprintf(stderr,
		_("\nUsage: xmmsd [options] ...\n\n"
		  "Options:\n"
		  "--------\n\n"
		  "-h, --help              Display this text and exit.\n"
		  "-b, --buffer-vis-data	Time (in milliseconds) to buffer vis data.\n"
        "-p, --pulse-rate        How many times a second to pulse status info.\n"
        "-v, --version           Print version number and exit.\n\n"));
	exit(0);
}

void parse_cmd_line(int argc, char **argv, gboolean remote)
{
	gint c, o;

   while ((c = getopt_long(argc, argv, "hb:p:v", long_options, &o)) != -1)
	{
		switch (c)
		{
			case 0:
				switch (o)
				{
					case 0:
						display_usage();
						break;
               case 1:
                  if(atoi(optarg) > 0)
                     mswait = atoi(optarg);
                  break;
               case 2:
                  if(atoi(optarg) > 0)
                     pulserate = atoi(optarg);
                  break;
					case 3:
						printf("%s %s\n", PACKAGE, VERSION);
						exit(0);
						break;
				}
				break;
			case 'h':
				display_usage();
				break;
			case 'b':
            if(atoi(optarg) > 0)
               mswait = atoi(optarg);
				break;
         case 'p':
            if(atoi(optarg) > 0)
               pulserate = atoi(optarg);
            break;
			case 'v':
				printf("%s %s\n", PACKAGE, VERSION);
				exit(0);
				break;
		}
	}
}

/* Signal Handlers */
void segfault_handler(int sig)
{
	printf("\nSegmentation fault in %s thread with pid %i.\n\n"
          "You've probably found a bug in XMMSd. Oops!\n\n"
          "Now attempting to shutdown gracefully (i.e. brace "
          "yourself to SIGTERM me!)...\n\n", which_thread(), getpid());
   raise(SIGTERM);
   exit(1);
}

void shutdown_xmms()
{
   //save_config();
   playlist_stop_get_info_thread();
   playlist_clear();
   //cleanup_plugins();
   printf("XMMS playback halted.\n");
}

void shutdown_handler(int sig)
{
   /* Shutdown everything - but only if we are the main thread. */
   if(pthread_equal(pthread_self(),get_main_thread()))
   {
      printf("Shutting down...\n");

      /* Tell the threads that we have stopped. */
      stop_running();

      /* Stop xmms playback, etc. */
      shutdown_xmms();

      /* Unmount both memory cards just incase. */
      umount_ps2_mc();
      
      /* Halt the buffer and play threads. */
      halt_buffer_thread();
      halt_play_thread();

      /* Close the open sockets. */
      drop_sockets();

      /* Close the command thread. */
      halt_command_thread();
      
      /* Free up the XML parser. */
      xml_comm_close();

      /* Shutdown the URL parser. */
      shutdown_url_parse();

      /* Close the status thread. */
      halt_status_thread();

      printf("Shutdown complete.\n");
      exit(sig);
   }
}

int main(int argc, char **argv)
{
   gchar *filename;
   int args[2], tmpvol;
   
   parse_cmd_line(argc, argv, FALSE);

   set_main_thread(pthread_self());

   /* Make thyself a daemon! */
   //daemon(0,1);

   /* Signal Handling */
   signal(SIGSEGV, segfault_handler);
   signal(SIGTERM, shutdown_handler);
   signal(SIGINT, shutdown_handler);
   
   /* XMMS initialization */
   g_thread_init(NULL);
   srand(time(NULL));
	read_config();
	//make_xmms_dir();
   init_plugins();
	//filename = g_strconcat(get_ps2_mc(), "/xmms.m3u", NULL);
   filename = g_strconcat(g_get_home_dir(), "/.xmms/xmms.m3u", NULL);
	playlist_load(filename);
	g_free(filename);
   umount_ps2_mc();
   mainwin_svis = create_svis();
   active_vis = create_vis();

	if (cfg.save_playlist_position)
		playlist_set_position(cfg.playlist_position);

   /* Initialize the URL parser. */
   init_url_parse();
   
   /* Start the status, pulse, play, and command threads. */
   args[0]=mswait;
   args[1]=pulserate;
   
   printf("\n\n"); //Just to make the initial console output easier to
                   //read.
   printf("Main Thread PID: %i\n",getpid());
   
   /* Start the threads that make us go! */
   start_play_thread(pulserate);
   start_buffer_thread(args);
   start_status_thread();
   start_command_thread();
   
   while(1){sleep(1);}
   
   return 0;
}
