/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "xmms.h"

static const gfloat vis_afalloff_speeds[] =
{0.34, 0.5, 1.0, 1.3, 1.6};
static const gfloat vis_pfalloff_speeds[] =
{1.2, 1.3, 1.4, 1.5, 1.6};
static const gint vis_redraw_delays[] =
{1, 2, 4, 8};
static const guint8 vis_scope_colors[] =
{21, 21, 20, 20, 19, 19, 18, 19, 19, 20, 20, 21, 21};

void vis_timeout_func(Vis * vis, guchar * data)
{
	static GTimer *timer = NULL;
	gulong micros = 9999999;
	gboolean falloff = FALSE;
	gint i;
   
	if (!timer)
	{
		timer = g_timer_new();
		g_timer_start(timer);
	}
	else
	{
		g_timer_elapsed(timer, &micros);
		if (micros > 14000)
			g_timer_reset(timer);

	}
   
	if (cfg.vis_type == VIS_ANALYZER)
	{
		if (micros > 14000)
         falloff = TRUE;
		if (data || falloff)
		{
			for (i = 0; i < 75; i++)
			{
				if (data && data[i] > vis->vs_data[i])
				{
					vis->vs_data[i] = data[i];
					if (vis->vs_data[i] > vis->vs_peak[i])
					{
                  vis->vs_peak[i] = vis->vs_data[i];
						vis->vs_peak_speed[i] = 0.01;

					}
					else if (vis->vs_peak[i] > 0.0)
					{
						vis->vs_peak[i] -= vis->vs_peak_speed[i];

                  vis->vs_peak_speed[i] *= vis_pfalloff_speeds[cfg.peaks_falloff];
						if (vis->vs_peak[i] < vis->vs_data[i])
							vis->vs_peak[i] = vis->vs_data[i];
						if (vis->vs_peak[i] < 0.0)
							vis->vs_peak[i] = 0.0;
					}
				}
				else if (falloff)
				{
					if (vis->vs_data[i] > 0.0)
					{
						vis->vs_data[i] -= vis_afalloff_speeds[cfg.analyzer_falloff];
                  
						if (vis->vs_data[i] < 0.0)
                     vis->vs_data[i] = 0.0;
                  
					}
					if (vis->vs_peak[i] > 0.0)
					{
                  
                  vis->vs_peak[i] -= vis->vs_peak_speed[i];
                  vis->vs_peak_speed[i] *= vis_pfalloff_speeds[cfg.peaks_falloff];
						if (vis->vs_peak[i] < vis->vs_data[i])
							vis->vs_peak[i] = vis->vs_data[i];
                  if (vis->vs_peak[i] < 0.0)
                     vis->vs_peak[i] = 0.0;
                  
               }
				}
			}
		}
	}
	else if (data)
	{
		for (i = 0; i < 75; i++)
			vis->vs_data[i] = data[i];
   }

	if (micros > 14000)
	{
		if (!vis->vs_refresh_delay)
		{
			vis->vs_refresh_delay = vis_redraw_delays[cfg.vis_refresh];
		}
		vis->vs_refresh_delay--;
   }
}

void vis_clear_data(Vis * vis)
{
	gint i;

	for (i = 0; i < 75; i++)
	{
		vis->vs_data[i] = (cfg.vis_type == VIS_SCOPE) ? 6 : 0;
		vis->vs_peak[i] = 0;
	}
}

void vis_set_doublesize(Vis * vis, gboolean doublesize)
{
	vis->vs_doublesize = doublesize;
}

Vis *create_vis()
{
	Vis *vis;

	vis = (Vis *) g_malloc0(sizeof (Vis));
	return vis;
}
