/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson, 4Front Technologies
 *  Copyright (C) 2001       Gray Carper, Manu Sporny (xRhino, Inc.)
 *
 *  Daemonization by xRhino, Inc. 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "xmms.h"

gboolean setting_volume = FALSE;
static gint balance;

void volume_set(gint newvol)
{
   gint vol;
   vol = CLAMP(newvol, 0, 100);
   adjust_volume(vol);
   setting_volume = FALSE;
   read_volume(VOLUME_SET);
}

void set_balance_left(gint left)
{
   gint b, bal;

   b = left-50;
   bal = CLAMP(b, -100, 100);
   
   if(bal > 0)
   {
      bal = -(bal*2);
   }
   else if(bal < 0)
   {
      bal = abs(bal)*2;
   }
   
   adjust_balance(bal);
   setting_volume = FALSE;
   read_volume(VOLUME_SET);
}

void set_balance_right(gint right)
{
   gint b, bal;

   b = right-50;
   bal = CLAMP(b, -100, 100);

   if(bal < 0)
   {
      bal = -(abs(bal)*2);
   }
   else if(bal > 0)
   {
      bal = bal*2;
   }
   
   adjust_balance(bal);
   setting_volume = FALSE;
   read_volume(VOLUME_SET);
}

void read_volume(gint when)
{
	int vl, vr, b, v;
	static int pvl = 0, pvr = 0, times = VOLSET_DISP_TIMES;
	static gboolean changing = FALSE;

	input_get_volume(&vl, &vr);
	if (when == VOLSET_STARTUP)
	{
		vl = CLAMP(vl, 0, 100);
		vr = CLAMP(vr, 0, 100);
		pvl = vl;
		pvr = vr;
		v = MAX(vl,vr);
		if(vl > vr)
			b = (gint) rint(((gdouble) vr / vl) * 100) - 100;
		else if(vl < vr)
			b = 100 - (gint) rint(((gdouble) vl / vr) * 100);
		else
			b = 0;

		balance = b;
		return;
	}

	else if (when == VOLSET_UPDATE)
	{
		if (vl == -1 || vr == -1)
			return;

		if (setting_volume)
		{
			pvl = vl;
			pvr = vr;
			return;
		}
		else if (pvr == vr && pvl == vl && changing)
		{
			if (times < VOLSET_DISP_TIMES)
				times++;
			else
			{
				changing = FALSE;
			}
		}
		else if (pvr != vr || pvl != vl)
		{
			gchar *tmp;
			
			v = MAX(vl,vr);
			if(vl > vr)
				b = (gint) rint(((gdouble) vr / vl) * 100) - 100;
			else if(vl < vr)
				b = 100 - (gint) rint(((gdouble) vl / vr) * 100);
			else
				b = 0;

			if (MAX(vl, vr) != MAX(pvl, pvr))
				tmp = g_strdup_printf(_("VOLUME: %d%%"), v);
			else
			{
				if (vl > vr)
				{
					tmp = g_strdup_printf(_("BALANCE: %d%% LEFT"), -b);
				}
				else if (vr == vl)
					tmp = g_strdup_printf(_("BALANCE: CENTER"));
				else /* (vl < vr) */
				{
					tmp = g_strdup_printf(_("BALANCE: %d%% RIGHT"), b);
				}
			}
			g_free(tmp);

			pvr = vr;
			pvl = vl;
			times = 0;
			changing = TRUE;

			/*
			 * Don't change the balance slider if the volume has been
			 * set to zero.  The balance can be anything, and our best
			 * guess is what is was before.
			 */
			if (v > 0)
			{
				balance = b;
			}
		}
	}
	else if(when == VOLUME_ADJUSTED)
	{
		pvl = vl;
		pvr = vr;
	}
	else if(when == VOLUME_SET)
	{
		times = 0;
		changing = TRUE;
		pvl = vl;
		pvr = vr;
	}
}

void adjust_volume(gint v)
{
        setting_volume = TRUE;
        if (balance < 0)
                input_set_volume(v, (v * (100 - abs(balance))) / 100);
        else if (balance > 0)
                input_set_volume((v * (100 - abs(balance))) / 100, v);
        else    
                input_set_volume(v, v);
}

void adjust_balance(gint b)
{
        gint v, pvl, pvr;
                
        setting_volume = TRUE;
        balance = b;
        input_get_volume(&pvl, &pvr);
        v = MAX(pvl, pvr);
        if (b < 0)
           input_set_volume(v, (int) rint(((100 + b) / 100.0) * v));
        else if (b == 0)   
                input_set_volume(v, v);
        else /* (b > 0) */
                input_set_volume((int) rint(((100 - b) / 100.0) * v), v);
}

void set_volume_diff(gint diff)
{
        gint vl, vr, vol;
        input_get_volume(&vl, &vr);
        vol = MAX(vl,vr);  
        vol = CLAMP(vol + diff, 0, 100);
        adjust_volume(vol);
        setting_volume = FALSE;
        read_volume(VOLUME_SET);
}

void set_balance_diff(gint diff)
{
        gint b;
        b = CLAMP(balance + diff, -100, 100);
        adjust_balance(b);
        setting_volume = FALSE;
        read_volume(VOLUME_SET);
}

gint get_volume()
{
   gint vl, vr;
   input_get_volume(&vl, &vr);
   return CLAMP(MAX(vl,vr), 0, 100);
}

gint get_balance_left()
{
   if(balance > 0) //Balance is toward the right
      return 50-(balance/2);
   else if(balance < 0) //Balance is toward the left
      return (abs(balance)/2)+50;
   else //Balance is even
      return 50;
}

gint get_balance_right()
{
   if(balance > 0) //Balance is toward the right
      return 100-(50-(balance/2));
   else if(balance < 0) //Balance is toward the left
      return 100-((abs(balance)/2)+50);
   else //Balance is even
      return 50;
}
