/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson, 4Front Technologies
 *  Copyright (C) 2001       Gray Carper, Manu Sporny (xRhino, Inc.)
 *
 *  Daemonization by xRhino, Inc. 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "xmms.h"
#include "xmlcomm.h"
#include "daemon.h"
#include <string.h>
#include <expat.h>

XML_Parser parser; //Our XML parser
char *character_data; //A place to temporarily keep the XML cdata
int closed = TRUE;
static pthread_mutex_t xml_mutex = PTHREAD_MUTEX_INITIALIZER;

/* Commands */
void play_command(const char **attr)
{
   char temp[50];
   int i;
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"action")) //Process the 'action'
                                    //attribute...
      {
         if(!strcmp(attr[i+1], "start")) //start playing
         {
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
            
            playlist_play();
            transmit_to_client(ALL_CLIENTS,temp);
            transmit_to_client(ALL_CLIENTS,"<play action=\"start\"/>");
         }
         else if(!strcmp(attr[i+1], "stop")) //stop playing
         {
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
            
            input_stop();
            transmit_to_client(ALL_CLIENTS,temp);
            transmit_to_client(ALL_CLIENTS,"<play action=\"stop\"/>");
         }
         else if(!strcmp(attr[i+1], "pause")) //toggle play pause
         {
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
            
            input_pause();
            transmit_to_client(ALL_CLIENTS,temp);
            transmit_to_client(ALL_CLIENTS,"<play action=\"pause\"/>");
         }
      }
   }
}

void volume_command(const char **attr)
{
   int i;
   char temp[100];
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"level")) //set the volume level
      {
         volume_set(atoi(attr[i+1]));
         sprintf(temp, "<volume level=\"%s\"/>", attr[i+1]);
         transmit_to_client(ALL_CLIENTS,temp);
      }
   }
}

void balance_command(const char **attr)
{
   int i;
   int left = -1;
   int right = -1;
   char temp[100];
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"left")) //set the left balance percent
      {
         left = atoi(attr[i+1]);
      }
      else if(!strcmp(attr[i],"right")) //set the right balance percent
      {
         right = atoi(attr[i+1]);
      }
   }
   if(right > -1 && left > -1)
   {
      set_balance_right(right);
      sprintf(temp, "<balance left=\"%i\" right=\"%i\"/>", left, right);
      transmit_to_client(ALL_CLIENTS,temp);
   }
   else if(right > -1 && left <= -1)
   {
      set_balance_right(right);
      sprintf(temp, "<balance right=\"%i\"/>", right);
      transmit_to_client(ALL_CLIENTS,temp);
   }
   else if(left > -1 && right <= -1)
   {
      set_balance_left(left);
      sprintf(temp, "<balance left=\"%i\"/>", left);
      transmit_to_client(ALL_CLIENTS,temp);
   }
}

void seek_command(const char **attr)
{
   int i;
   char temp[100];

   /* Only seek if we aren't a webstream. */
   if(strstr(playlist_get_filename(get_playlist_position()),
             "http://") == NULL)
   {
      for(i=0;attr[i];i+=2) //For each attribute in the list...
      {
         if(!strcmp(attr[i],"time")) //seek to this time index (in seconds)
         {
            input_seek(atoi(attr[i+1]));
            sprintf(temp, "<seek time=\"%s\"/>", attr[i+1]);
            transmit_to_client(ALL_CLIENTS,temp);
         }
         else if(!strcmp(attr[i],"forward")) //seek forward by the given
         {                                   //seconds
            input_seek((input_get_time()/1000)+atoi(attr[i+1]));
            sprintf(temp, "<seek time=\"%i\"/>", input_get_time()/1000);
            transmit_to_client(ALL_CLIENTS,temp);
         }
         else if(!strcmp(attr[i],"backward")) //seek forward by the given
         {                                    //seconds
            input_seek((input_get_time()/1000)-atoi(attr[i+1]));
            sprintf(temp, "<seek time=\"%i\"/>", input_get_time()/1000);
            transmit_to_client(ALL_CLIENTS,temp);
         }
      }
   }
}

void feed_command(const char **attr)
{
   int i, index;
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"index")) //return info for the feed at this index
      {
         index = atoi(attr[i+1]);

         if(index >= 0 && index < get_playlist_length())
         {
            send_feed_info(ALL_CLIENTS,index);
         }
      }
   }
   if(i==0) //If there were no attributes.      
   {
      /* Send back the list of feeds in the playlist. */
      send_playlist(ALL_CLIENTS);
   }
}

void playlist_command(const char **attr)
{
   int i;
   int action = -1;
   int index = -1;
   char temp[1024];

   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"action"))
      {
         if(!strcmp(attr[i+1], "next")) //next feed in playlist
         {
            playlist_next();
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
            transmit_to_client(ALL_CLIENTS,temp);
         }
         else if(!strcmp(attr[i+1], "previous")) //previous feed in playlist
         {
            playlist_prev();
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
            transmit_to_client(ALL_CLIENTS,temp);
         }
         
         else if(!strcmp(attr[i+1], "add")) //add  new feed to playlist
         {
            action = 1;
         }
         else if(!strcmp(attr[i+1], "remove")) //remove feed from playlist
         {
            action = 2;
         }
         else if(!strcmp(attr[i+1], "jump")) //jump to feed in playlist
         {
            action = 3;
         }
         else if(!strcmp(attr[i+1], "save")) //save current playlist
         {
            action = 4;
         }
         else if(!strcmp(attr[i+1], "load")) //load playlist
         {
            action = 5;
         }
         else if(!strcmp(attr[i+1], "clear")) //clear playlist
         {
            playlist_clear();
            transmit_to_client(ALL_CLIENTS,"<playlist action=\"clear\"/>");
         }
      }
      else if(!strcmp(attr[i],"filename"))
      {
         strcpy(character_data,attr[i+1]);
      }
      else if(!strcmp(attr[i],"index"))
      {
         index = atoi(attr[i+1]);
      }
   }

   /* Now that we have the attributes, execute the command. */
   switch(action)
   {
      case 1: //Add to the playlist
         if(character_data)
         {
            i = get_playlist_length();
            
            if(strstr(character_data, "http://") != NULL)
               playlist_add_url_string(character_data);
            else
               playlist_add(character_data);
            send_feed_info(ALL_CLIENTS,get_playlist_length()-1);
            if(i == 0)
            {
               sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
                    get_playlist_position());
               transmit_to_client(ALL_CLIENTS,temp);
            }
         }
         break;
      case 2: //Remove from the playlist
         if(index >= 0 && index < get_playlist_length())
         {
            playlist_delete_index(index);
            sprintf(temp, "<playlist action=\"remove\" index=\"%i\"/>", index);
            transmit_to_client(ALL_CLIENTS,temp);
         }
         break;
      case 3: //Jump to a position in the playlist
         if(index >= 0 && index < get_playlist_length())
         {
            playlist_set_position(index);
            sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>", index);
            transmit_to_client(ALL_CLIENTS,temp);
            send_feed_info(ALL_CLIENTS,index); //So the clients learn about the feed
         }
         break;
      case 4: //Save the server's current playlist
         if(character_data)
         {
            playlist_save(character_data);
            sprintf(temp, "<playlist action=\"save\" filename=\"%s\"/>",
                    character_data);
            transmit_to_client(ALL_CLIENTS,temp);
         }
         break;
      case 5: //Load a playlist into the server
         if(character_data)
         {
            playlist_load(character_data);
            sprintf(temp, "<playlist action=\"load\" filename=\"%s\"/>",
                    character_data);
            transmit_to_client(ALL_CLIENTS,temp);
         }
         break;
   }
}

void shuffle_command(const char **attr)
{
   int i;
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"action"))
      {
         if(!strcmp(attr[i+1], "start")) //enable random play
         {
            shuffle_push(TRUE);
            transmit_to_client(ALL_CLIENTS,"<shuffle action=\"start\"/>");
         }
         else if(!strcmp(attr[i+1], "stop")) //disable random play
         {
            shuffle_push(FALSE);
            transmit_to_client(ALL_CLIENTS,"<shuffle action=\"stop\"/>");
         }
      }
   }
}

void repeat_command(const char **attr)
{
   int i;
   
   for(i=0;attr[i];i+=2) //For each attribute in the list...
   {
      if(!strcmp(attr[i],"action"))
      {
         if(!strcmp(attr[i+1], "start")) //enable repeat
         {
            repeat_push(TRUE);
            transmit_to_client(ALL_CLIENTS,"<repeat action=\"start\"/>");
         }
         else if(!strcmp(attr[i+1], "stop")) //disable repeat
         {
            repeat_push(FALSE);
            transmit_to_client(ALL_CLIENTS,"<repeat action=\"stop\"/>");
         }         
      }
   }
}

/* Command Handlers */
void handle_command_start(const char *command, const char **attr)
{
   /* Handle the 'play' command. */
   if(!strcmp(command, "play"))
   {
      play_command(attr);
   }

   /* Handle the 'volume' command. */
   if(!strcmp(command, "volume"))
   {
      volume_command(attr);
   }

   /* Handle the 'balance' command. */
   if(!strcmp(command, "balance"))
   {
      balance_command(attr);
   }

   /* Handle the 'seek' command. */
   if(!strcmp(command, "seek"))
   {
      seek_command(attr);
   }

   /* Handle the 'feed' command. */
   if(!strcmp(command, "feed"))
   {
      feed_command(attr);
   }

   /* Handle the 'playlist' command. */
   if(!strcmp(command, "playlist"))
   {
      playlist_command(attr);
   }

   /* Handle the 'shuffle' command. */
   if(!strcmp(command, "shuffle"))
   {
      shuffle_command(attr);
   }

   /* Handle the 'repeat' command. */
   if(!strcmp(command, "repeat"))
   {
      repeat_command(attr);
   }
}

void handle_command_end(const char *command)
{}

/* XML Handlers */
void xml_start_handler(void *data, const char *command, const char **attr)
{
   /* Initialize our temporary data holder. */
   memset(character_data,0x0,MAX_MSG);

   /* Handle start-tag command. */
   handle_command_start(command, attr);
}

void xml_character_handler(void *userData, const char *s, int len)
{
   /* Grab the character data. */
   strncat(character_data,s,len);
}

void xml_end_handler(void *data, const char *command)
{
   /* Handle end-tag command. */
   handle_command_end(command);
}

int xml_parse_this(char *parseme, int isFinal)
{
   int retval;
   char temp[350];
   
   retval = XML_Parse(parser, parseme, strlen(parseme), isFinal);
   
   if (retval == 0)
   { 
      sprintf(temp, "<error>Err %i: Parse error at line %d:%s</error>",
              XML_GetErrorCode(parser),
              XML_GetCurrentLineNumber(parser),
              XML_ErrorString(XML_GetErrorCode(parser)));
      printf("Following error produced from '%s' tag...\n",parseme);
      printf("%s\n",temp);
      //transmit_to_client(ALL_CLIENTS,temp);
   }
   else
   {
      printf("Successfully parsed '%s' tag.\n",parseme);
   }
   
   return retval;
}


void xml_comm_init()
{
   X_LOCK();
   if(closed)
   {
      closed = FALSE; //Keep track of our state.
      
      /* Allocate our parser. */
      parser = XML_ParserCreate(NULL);
      character_data = (char*) malloc(MAX_MSG);

      /* Setup the needed handlers. */
      XML_SetElementHandler(parser, xml_start_handler, xml_end_handler);
      XML_SetCharacterDataHandler(parser, xml_character_handler);
   }
   X_UNLOCK();
}

void xml_comm_close()
{
   X_LOCK();
   if(!closed)
   {
      closed = TRUE; //Keep track of our state.
      
      /* Cleanup our temporary data holder. */
      free(character_data);
      
      /* Cleanup the parser. */
      XML_ParserFree(parser); //-- this used to cause a segfault, watch it.
   }
   X_UNLOCK();
}

void xml_comm_reset()
{
   /* Reset the parser. */
   xml_comm_close();
   xml_comm_init();
}

void send_playlist(int sd)
{
   int i;
 
   /* Send the index and feed title for all feeds in the
      playlist. Note: If no id3 title is available, the
      filename is sent in it's place. */
   for(i=0;i<get_playlist_length();i++)
   {
      X_LOCK();
      send_feed_info(sd, i);
      X_UNLOCK();
   }
}

void send_xmlheader(int sd)
{
   transmit_to_client(sd, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
   transmit_to_client(sd, "<!DOCTYPE xmmscomm SYSTEM \"xmmscomm.dtd\">");
   transmit_to_client(sd, "<xmmscomm>");
}

void send_feed_info(int sd, int index)
{
   char temp[2048];
   gchar *title, *filename;
   gint length;
   FileInfo info;
   
   filename = playlist_get_filename(index);
   input_file_info(filename,&info);
   input_get_song_info(filename, &title, &length);
   sprintf(temp,"<feed index=\"%i\" source=\"%s\" bitrate=\"%i\" "
           "samplerate=\"%i\" filesize=\"%i\" channels=\"%s\" "
           "frames=\"%i\" title=\"%s\" artist=\"%s\" album=\"%s\" "
           "comment=\"%s\" year=\"%s\" track=\"%s\" genre=\"%s\" "
           "ep=\"%s\" copyright=\"%s\" original=\"%s\" "
           "emphasis=\"%s\" length=\"%i\" station=\"%s\"/>", index,
           encode_url(playlist_get_filename(index)), info.bitrate,
           info.sampling_frequency,info.filesize,info.channels,
           info.frames,encode_url(info.title),
           encode_url(info.artist),encode_url(info.album),
           encode_url(info.comment),info.year,"",
           info.genre, info.error_protection,info.copyright,
           info.original, info.emphasis, length/1000,
           encode_url(info.station));
   transmit_to_client(sd,temp);
   g_free(title);
   g_free(filename);
}

void send_state(int sd)
{
   char temp[500];

   /* Send all the current state info to the client. */

   /* Play state */
   if(get_input_playing())
      transmit_to_client(sd,"<play action=\"start\"/>");
   else if(get_input_paused())
      transmit_to_client(sd,"<play action=\"pause\"/>");
   else
      transmit_to_client(sd,"<play action=\"stop\"/>");

   /* Volume state */
   memset(temp,0x0,500);
   sprintf(temp, "<volume level=\"%i\"/>", get_volume());
   transmit_to_client(sd,temp);
   
   /* Balance state */
   memset(temp,0x0,500);
   sprintf(temp, "<balance left=\"%i\" right=\"%i\"/>",
           get_balance_left(),get_balance_right());
   transmit_to_client(sd,temp);

   /* Jump to currently playing feed index */
   memset(temp,0x0,500);
   sprintf(temp, "<playlist action=\"jump\" index=\"%i\"/>",
           get_playlist_position());
   transmit_to_client(sd,temp);

   /* Time state */
   memset(temp,0x0,500);
   sprintf(temp, "<seek time=\"%i\"/>", input_get_time()/1000);
   transmit_to_client(sd,temp);

   /* Shuffle state */
   if(is_shuffle())
      transmit_to_client(sd,"<shuffle action=\"start\">");
   else
      transmit_to_client(sd,"<shuffle action=\"stop\">");

   /* Repeat state */
   if(is_repeat())
      transmit_to_client(sd,"<repeat action=\"start\">");
   else
      transmit_to_client(sd,"<repeat action=\"stop\">");
}
